# -*- coding: utf-8 -*-

# Based on code from: Jannis Leidal's blog 2008 (http://jannisleidel.com/),
# Copyright (C) 2010: Sindre Røkenes Myren,

import operator

from django.db import models
from django.http import HttpResponse, HttpResponseNotFound
from django.utils.encoding import smart_str


def fk_search(request, q=None):
	"""Searches in the fields of the given related model and returns the
	result as a simple string to be used by the jQuery Autocomplete plugin

	"""
	query = request.GET.get('q', None)
	app_label = request.GET.get('app_label', None)
	model_name = request.GET.get('model_name', None)
	search_fields = request.GET.get('search_fields', None)
	if search_fields and app_label and model_name and query:
		def construct_search(field_name):
			# use different lookup methods depending on the notation
			if field_name.startswith('^'):
				return "%s__istartswith" % field_name[1:]
			elif field_name.startswith('='):
				return "%s__iexact" % field_name[1:]
			elif field_name.startswith('@'):
				return "%s__search" % field_name[1:]
			else:
				return "%s__icontains" % field_name

		model = models.get_model(app_label, model_name)
		qs = model._default_manager.all()
		for bit in query.split():
			or_queries = [models.Q(**{construct_search(
				smart_str(field_name)): smart_str(bit)})
					for field_name in search_fields.split(',')]
			other_qs = models.query.QuerySet(model)
			other_qs.dup_select_related(qs)
			other_qs = other_qs.filter(reduce(operator.or_, or_queries))
			qs = qs & other_qs
		data = ''.join([u'%s|%s\n' % (f.__unicode__(), f.pk) for f in qs])
		return HttpResponse(data)
	return HttpResponseNotFound()



# -*- coding: utf-8 -*-

# Based on code from: Jannis Leidal, 2008 (http://jannisleidel.com/),
# Copyright (C) 2010: Sindre Røkenes Myren,

# This file is part of KiKrit wich is distrebuted under GPLv3. See the file
# COPYING.txt for more details.

import operator
from django.db import models
from django.http import HttpResponse, HttpResponseNotFound
from django.contrib import admin
from django.utils.encoding import smart_str

from jquery_widgets.widgets import ForeignKeySearchInput


class ExtendedModelAdmin(admin.ModelAdmin):
	"""Extends the normal ModalAdmin with an alternative widget for FoeignKey,
	if you provide it with an aditional attribute related_search_fields.

	Example:

	related_search_fields = {
		'user': ('username', 'email'),
	}

	"""

	def formfield_for_dbfield(self, db_field, **kwargs):
		""" Overrides the default widget for Foreignkey fields if they are
		specified in the related_search_fields class attribute.

		"""
		if isinstance(db_field, models.ForeignKey) and \
				db_field.name in self.related_search_fields:
			kwargs['widget'] = ForeignKeySearchInput(db_field.rel,
									self.related_search_fields[db_field.name])
		return super(ExtendedModelAdmin, self).formfield_for_dbfield(db_field,
				**kwargs)



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('',
	url(r'^fk-search/$', 'jquery_widgets.views.fk_search',
		name="fk-search"),
	url(r'^fk-search/(.*)$', 'jquery_widgets.views.fk_search',
		name="fk-search-q"),
)



# -*- coding: utf-8 -*-

# Based on code from Jannis Leidel's blog 2008 (http://jannisleidel.com/)
# Copyright (C) 2010: Sindre Røkenes Myren,

from django import VERSION, forms
from django.conf import settings
from django.utils.safestring import mark_safe
from django.utils.text import truncate_words
from django.template.loader import render_to_string

# Get JQuery URL from settings, or fallback to default value:
JS_BASE = tuple() # Basic tuple of javascripts to extend.
try:
	JQUERY_URL = settings.JQUERY_URL
except AttributeError:
	JQUERY_URL = 'js/jquery.min.js'

	# Django 1.2 and later include a version of jQury in contrib.admin:
	if VERSION[:2] >= (1, 2):
		try:
			JQUERY_URL = settings.ADMIN_MEDIA_PREFIX + JQUERY_URL
			# Django admin keep it's jQuery in the 'django.jQuey' namespace,
			# which is kind of anoying. If we are to use jQury extensions
			# without modifications, we must make sure jQuery is put back into
			# the global namespace:
			JS_BASE = (
				settings.ADMIN_MEDIA_PREFIX+'jquery.init.js',
				'jquery_widgets/js/django.jquery.reinit.js',
			)
		except AttributeError:
			JQUERY_URL = 'js/jquery.min.js'

JS_BASE = (JQUERY_URL,) + JS_BASE



class ForeignKeySearchInput(forms.HiddenInput):
	""" A Widget for displaying ForeignKeys in an autocomplete search input
	instead in a ``select`` box.
	"""
	class Media:
		css = {
			'all': ('jquery_widgets/css/jquery.autocomplete.css',)
		}
		js = JS_BASE + (
			'jquery_widgets/js/jquery.autocomplete.min.js',
		)

	def label_for_value(self, value):
		key = self.rel.get_related_field().name
		obj = self.rel.to._default_manager.get(**{key: value})
		return truncate_words(obj, 14)

	def __init__(self, rel, search_fields, attrs=None):
		self.rel = rel
		self.search_fields = search_fields
		super(ForeignKeySearchInput, self).__init__(attrs)

	def render(self, name, value, attrs={}):
		rendered = super(ForeignKeySearchInput, self).render(name, value, attrs)
		if value:
			label = self.label_for_value(value)
		else:
			label = u''

		d = {
			'rendered': rendered,
			'current_app': 'jquery_widgets',
			'search_fields': ','.join(self.search_fields),
			'admin_media_prefix': settings.ADMIN_MEDIA_PREFIX,
			'model_name': self.rel.to._meta.module_name,
			'app_label': self.rel.to._meta.app_label,
			'label': label,
			'name': name,
		}
		return render_to_string('jquery_widgets/fk_search_input.html', d)



# -*- coding: utf-8 -*-
from django import forms
from django.conf import settings

from jquey_widgets.widgets import ForeignKeySearchInput


class ForeignKeySearchField(forms.fields.ChoiceField):
	"""Form field to select a model for a ForeignKey db field"""
	pass # Fix this field later



# -*- coding: utf-8 -*-
#!/usr/bin/env python

from distutils.core import setup
VERSION = '0.1.1'

setup(name='django-jquery-widgets',
	version=VERSION,
	description='jQuery-powered widgets and fields for Django',
	author='Sindre Røkenes Myren',
	author_email='smyrman@gmail.com',
	url='http://github.com/smyrman/django-jquery-widgets',
	install_requires='django >= 1.1',
	download_url='http://github.com/smyrman/django-jquery-widgets/tarball/%s'%VERSION,
	packages=['jquery_widgets'],
	package_data={'jquery_widgets': ['media/js/*.js', 'media/css/*.css']},
)
